#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "Booleans.h"

/*{{{  conversion functions */

ATerm SDFstringToChars(const char *str)
{
  int len = strlen(str);
  int i;
  ATermList result = ATempty;

  for (i = len - 1; i >= 0; i--) {
    result = ATinsert(result, (ATerm) ATmakeInt(str[i]));
  }

  return (ATerm) result;
}

ATerm SDFbyteToChar(char ch)
{
    return (ATerm) ATmakeInt(ch);
}

char *SDFcharsToString(ATerm arg)
{
  ATermList list = (ATermList) arg;
  int len = ATgetLength(list);
  int i;
  char *str;

  str = (char *) malloc(len+1);
  if (str == NULL) {
      return NULL;
  }

  for (i = 0; !ATisEmpty(list); list = ATgetNext(list), i++) {
    str[i] = (char) ATgetInt((ATermInt) ATgetFirst(list));
  }
  str[i] = '\0';

  return str;
}

char SDFcharToByte(ATerm arg)
{
    return (char) ATgetInt((ATermInt) arg);
}


/*}}}  */

/*{{{  typedefs */

typedef struct ATerm _SDFOptLayout;
typedef struct ATerm _SDFLayout;
typedef struct ATerm _SDFLexLayoutList;
typedef struct ATerm _SDFStart;
typedef struct ATerm _SDFLexLayout;
typedef struct ATerm _SDFBool;
typedef struct ATerm _SDFBoolList;
typedef struct ATerm _SDFBoolElems;

/*}}}  */

/*{{{  void SDFinitBooleansApi(void) */

void SDFinitBooleansApi(void)
{
  init_Booleans_dict();
}

/*}}}  */

/*{{{  (un)protect functions */

void SDFprotectOptLayout(SDFOptLayout *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFunprotectOptLayout(SDFOptLayout *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void SDFprotectLayout(SDFLayout *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFunprotectLayout(SDFLayout *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void SDFprotectLexLayoutList(SDFLexLayoutList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFunprotectLexLayoutList(SDFLexLayoutList *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void SDFprotectStart(SDFStart *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFunprotectStart(SDFStart *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void SDFprotectLexLayout(SDFLexLayout *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFunprotectLexLayout(SDFLexLayout *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void SDFprotectBool(SDFBool *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFunprotectBool(SDFBool *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void SDFprotectBoolList(SDFBoolList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFunprotectBoolList(SDFBoolList *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void SDFprotectBoolElems(SDFBoolElems *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFunprotectBoolElems(SDFBoolElems *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}


/*}}}  */
/*{{{  term conversion functions */

/*{{{  SDFOptLayout SDFOptLayoutFromTerm(ATerm t) */

SDFOptLayout SDFOptLayoutFromTerm(ATerm t)
{
  return (SDFOptLayout)t;
}

/*}}}  */
/*{{{  ATerm SDFOptLayoutToTerm(SDFOptLayout arg) */

ATerm SDFOptLayoutToTerm(SDFOptLayout arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFLayout SDFLayoutFromTerm(ATerm t) */

SDFLayout SDFLayoutFromTerm(ATerm t)
{
  return (SDFLayout)t;
}

/*}}}  */
/*{{{  ATerm SDFLayoutToTerm(SDFLayout arg) */

ATerm SDFLayoutToTerm(SDFLayout arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFLexLayoutList SDFLexLayoutListFromTerm(ATerm t) */

SDFLexLayoutList SDFLexLayoutListFromTerm(ATerm t)
{
  return (SDFLexLayoutList)t;
}

/*}}}  */
/*{{{  ATerm SDFLexLayoutListToTerm(SDFLexLayoutList arg) */

ATerm SDFLexLayoutListToTerm(SDFLexLayoutList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFStart SDFStartFromTerm(ATerm t) */

SDFStart SDFStartFromTerm(ATerm t)
{
  return (SDFStart)t;
}

/*}}}  */
/*{{{  ATerm SDFStartToTerm(SDFStart arg) */

ATerm SDFStartToTerm(SDFStart arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFLexLayout SDFLexLayoutFromTerm(ATerm t) */

SDFLexLayout SDFLexLayoutFromTerm(ATerm t)
{
  return (SDFLexLayout)t;
}

/*}}}  */
/*{{{  ATerm SDFLexLayoutToTerm(SDFLexLayout arg) */

ATerm SDFLexLayoutToTerm(SDFLexLayout arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFBool SDFBoolFromTerm(ATerm t) */

SDFBool SDFBoolFromTerm(ATerm t)
{
  return (SDFBool)t;
}

/*}}}  */
/*{{{  ATerm SDFBoolToTerm(SDFBool arg) */

ATerm SDFBoolToTerm(SDFBool arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFBoolList SDFBoolListFromTerm(ATerm t) */

SDFBoolList SDFBoolListFromTerm(ATerm t)
{
  return (SDFBoolList)t;
}

/*}}}  */
/*{{{  ATerm SDFBoolListToTerm(SDFBoolList arg) */

ATerm SDFBoolListToTerm(SDFBoolList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFBoolElems SDFBoolElemsFromTerm(ATerm t) */

SDFBoolElems SDFBoolElemsFromTerm(ATerm t)
{
  return (SDFBoolElems)t;
}

/*}}}  */
/*{{{  ATerm SDFBoolElemsToTerm(SDFBoolElems arg) */

ATerm SDFBoolElemsToTerm(SDFBoolElems arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  list functions */

int SDFgetLexLayoutListLength (SDFLexLayoutList arg) {
  return ATgetLength((ATermList) arg);
}
SDFLexLayoutList SDFreverseLexLayoutList(SDFLexLayoutList arg) {
  return (SDFLexLayoutList) ATreverse((ATermList) arg);
}
SDFLexLayoutList SDFappendLexLayoutList(SDFLexLayoutList arg, SDFLexLayout elem) {
  return (SDFLexLayoutList) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
SDFLexLayoutList SDFconcatLexLayoutList(SDFLexLayoutList arg0, SDFLexLayoutList arg1) {
  return (SDFLexLayoutList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDFLexLayoutList SDFsliceLexLayoutList(SDFLexLayoutList arg, int start, int end) {
  return (SDFLexLayoutList) ATgetSlice((ATermList) arg, start, end);
}
SDFLexLayout SDFgetLexLayoutListLexLayoutAt(SDFLexLayoutList arg, int index) {
 return (SDFLexLayout)ATelementAt((ATermList) arg,index);
}
SDFLexLayoutList SDFreplaceLexLayoutListLexLayoutAt(SDFLexLayoutList arg, SDFLexLayout elem, int index) {
 return (SDFLexLayoutList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
SDFLexLayoutList SDFmakeLexLayoutList2(SDFLexLayout elem1, SDFLexLayout elem2) {
  return (SDFLexLayoutList) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
SDFLexLayoutList SDFmakeLexLayoutList3(SDFLexLayout elem1, SDFLexLayout elem2, SDFLexLayout elem3) {
  return (SDFLexLayoutList) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
SDFLexLayoutList SDFmakeLexLayoutList4(SDFLexLayout elem1, SDFLexLayout elem2, SDFLexLayout elem3, SDFLexLayout elem4) {
  return (SDFLexLayoutList) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
SDFLexLayoutList SDFmakeLexLayoutList5(SDFLexLayout elem1, SDFLexLayout elem2, SDFLexLayout elem3, SDFLexLayout elem4, SDFLexLayout elem5) {
  return (SDFLexLayoutList) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
SDFLexLayoutList SDFmakeLexLayoutList6(SDFLexLayout elem1, SDFLexLayout elem2, SDFLexLayout elem3, SDFLexLayout elem4, SDFLexLayout elem5, SDFLexLayout elem6) {
  return (SDFLexLayoutList) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}
int SDFgetBoolElemsLength (SDFBoolElems arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDFBoolElems SDFreverseBoolElems(SDFBoolElems arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDFBoolElems) result;
}
SDFBoolElems SDFappendBoolElems(SDFBoolElems arg0, SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool arg1) {
  return SDFconcatBoolElems(arg0, wsAfterHead, wsAfterSep, SDFmakeBoolElemsSingle(arg1));
}
SDFBoolElems SDFconcatBoolElems(SDFBoolElems arg0, SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBoolElems arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDFmakeBoolElemsMany((SDFBool)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDFBoolElems) ATgetNext((ATermList) arg1);
  return (SDFBoolElems) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDFBoolElems SDFsliceBoolElems(SDFBoolElems arg, int start, int end) {
  return (SDFBoolElems) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDFBool SDFgetBoolElemsBoolAt(SDFBoolElems arg, int index) {
 return (SDFBool)ATelementAt((ATermList) arg,index * 4);
}
SDFBoolElems SDFreplaceBoolElemsBoolAt(SDFBoolElems arg, SDFBool elem, int index) {
 return (SDFBoolElems) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDFBoolElems SDFmakeBoolElems2(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElemsSingle(elem2));
}
SDFBoolElems SDFmakeBoolElems3(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2, SDFBool elem3) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElems2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDFBoolElems SDFmakeBoolElems4(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2, SDFBool elem3, SDFBool elem4) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElems3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDFBoolElems SDFmakeBoolElems5(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2, SDFBool elem3, SDFBool elem4, SDFBool elem5) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElems4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDFBoolElems SDFmakeBoolElems6(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2, SDFBool elem3, SDFBool elem4, SDFBool elem5, SDFBool elem6) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElems5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}

/*}}}  */
/*{{{  constructors */

/*{{{  SDFOptLayout SDFmakeOptLayoutAbsent(void) */

SDFOptLayout SDFmakeOptLayoutAbsent(void)
{
  return (SDFOptLayout)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATempty, (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4))), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATempty);
}

/*}}}  */
/*{{{  SDFOptLayout SDFmakeOptLayoutPresent(SDFLayout layout) */

SDFOptLayout SDFmakeOptLayoutPresent(SDFLayout layout)
{
  return (SDFOptLayout)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun4))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4))), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATmakeList1((ATerm) layout));
}

/*}}}  */
/*{{{  SDFLayout SDFmakeLayoutLexToCf(SDFLexLayoutList list) */

SDFLayout SDFmakeLayoutLexToCf(SDFLexLayoutList list)
{
  return (SDFLayout)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun6, (ATerm)ATmakeAppl1(SDFafun7, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun4)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl1(SDFafun8, (ATerm)ATmakeAppl1(SDFafun6, (ATerm)ATmakeAppl1(SDFafun7, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDFLexLayoutList SDFmakeLexLayoutListEmpty(void) */

SDFLexLayoutList SDFmakeLexLayoutListEmpty(void)
{
  return (SDFLexLayoutList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDFLexLayoutList SDFmakeLexLayoutListSingle(SDFLexLayout head) */

SDFLexLayoutList SDFmakeLexLayoutListSingle(SDFLexLayout head)
{
  return (SDFLexLayoutList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDFLexLayoutList SDFmakeLexLayoutListMany(SDFLexLayout head, SDFLexLayoutList tail) */

SDFLexLayoutList SDFmakeLexLayoutListMany(SDFLexLayout head, SDFLexLayoutList tail)
{
  return (SDFLexLayoutList)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  SDFStart SDFmakeStartBool(SDFOptLayout wsBefore, SDFBool topBool, SDFOptLayout wsAfter, int ambCnt) */

SDFStart SDFmakeStartBool(SDFOptLayout wsBefore, SDFBool topBool, SDFOptLayout wsAfter, int ambCnt)
{
  return (SDFStart)(ATerm)ATmakeAppl2(SDFafun9, (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun12)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) wsAfter), (ATerm) topBool), (ATerm) wsBefore)), (ATerm) (ATerm) ATmakeInt(ambCnt));
}

/*}}}  */
/*{{{  SDFStart SDFmakeStartBoolList(SDFOptLayout wsBefore, SDFBoolList topBoolList, SDFOptLayout wsAfter, int ambCnt) */

SDFStart SDFmakeStartBoolList(SDFOptLayout wsBefore, SDFBoolList topBoolList, SDFOptLayout wsAfter, int ambCnt)
{
  return (SDFStart)(ATerm)ATmakeAppl2(SDFafun9, (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun13)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun12)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) wsAfter), (ATerm) topBoolList), (ATerm) wsBefore)), (ATerm) (ATerm) ATmakeInt(ambCnt));
}

/*}}}  */
/*{{{  SDFLexLayout SDFmakeLexLayoutDefault(char ch) */

SDFLexLayout SDFmakeLexLayoutDefault(char ch)
{
  return (SDFLexLayout)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(32)), (ATerm)ATmakeAppl2(SDFafun15, (ATerm)ATmakeInt(9), (ATerm)ATmakeInt(10))))), (ATerm)ATmakeAppl1(SDFafun6, (ATerm)ATmakeAppl0(SDFafun4)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATmakeList1((ATerm) ((ATerm) SDFbyteToChar(ch))));
}

/*}}}  */
/*{{{  SDFBool SDFmakeBoolTrue(void) */

SDFBool SDFmakeBoolTrue(void)
{
  return (SDFBool)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun17))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11))), (ATerm)ATmakeAppl1(SDFafun18, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun19, (ATerm)ATmakeAppl1(SDFafun20, (ATerm)ATmakeAppl0(SDFafun17)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(117)))), (ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun17)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(117)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(116)))));
}

/*}}}  */
/*{{{  SDFBool SDFmakeBoolFalse(void) */

SDFBool SDFmakeBoolFalse(void)
{
  return (SDFBool)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun21))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11))), (ATerm)ATmakeAppl1(SDFafun18, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun19, (ATerm)ATmakeAppl1(SDFafun20, (ATerm)ATmakeAppl0(SDFafun21)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(102)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun21)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(102)))));
}

/*}}}  */
/*{{{  SDFBool SDFmakeBoolAnd(SDFBool left, SDFOptLayout wsAfterLeft, SDFOptLayout wsAfterAmp, SDFBool right) */

SDFBool SDFmakeBoolAnd(SDFBool left, SDFOptLayout wsAfterLeft, SDFOptLayout wsAfterAmp, SDFBool right)
{
  return (SDFBool)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun22))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11))), (ATerm)ATmakeAppl1(SDFafun18, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun23, (ATerm)ATmakeAppl0(SDFafun24))), (ATerm)ATmakeAppl1(SDFafun19, (ATerm)ATmakeAppl1(SDFafun20, (ATerm)ATmakeAppl0(SDFafun25)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterAmp), (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(38)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun22)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(38)))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDFBool SDFmakeBoolOr(SDFBool left, SDFOptLayout wsAfterLeft, SDFOptLayout wsAfterBar, SDFBool right) */

SDFBool SDFmakeBoolOr(SDFBool left, SDFOptLayout wsAfterLeft, SDFOptLayout wsAfterBar, SDFBool right)
{
  return (SDFBool)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun26))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11))), (ATerm)ATmakeAppl1(SDFafun18, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun23, (ATerm)ATmakeAppl0(SDFafun24))), (ATerm)ATmakeAppl1(SDFafun19, (ATerm)ATmakeAppl1(SDFafun20, (ATerm)ATmakeAppl0(SDFafun27)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterBar), (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(124)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun26)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(124)))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDFBoolList SDFmakeBoolListElems(SDFOptLayout wsAfterBracketOpen, SDFBoolElems elems, SDFOptLayout wsAfterElems) */

SDFBoolList SDFmakeBoolListElems(SDFOptLayout wsAfterBracketOpen, SDFBoolElems elems, SDFOptLayout wsAfterElems)
{
  return (SDFBoolList)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun28))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl2(SDFafun29, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11)), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun30))))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun3, (ATerm)ATmakeAppl0(SDFafun4)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun31))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun13))), (ATerm)ATmakeAppl1(SDFafun18, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun19, (ATerm)ATmakeAppl1(SDFafun20, (ATerm)ATmakeAppl0(SDFafun32)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun28)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm) wsAfterElems), (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl1(SDFafun8, (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl2(SDFafun29, (ATerm)ATmakeAppl1(SDFafun10, (ATerm)ATmakeAppl0(SDFafun11)), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun30))))), (ATerm) elems)), (ATerm) wsAfterBracketOpen), (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun31)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))));
}

/*}}}  */
/*{{{  SDFBoolElems SDFmakeBoolElemsEmpty(void) */

SDFBoolElems SDFmakeBoolElemsEmpty(void)
{
  return (SDFBoolElems)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDFBoolElems SDFmakeBoolElemsSingle(SDFBool head) */

SDFBoolElems SDFmakeBoolElemsSingle(SDFBool head)
{
  return (SDFBoolElems)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDFBoolElems SDFmakeBoolElemsMany(SDFBool head, SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBoolElems tail) */

SDFBoolElems SDFmakeBoolElemsMany(SDFBool head, SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBoolElems tail)
{
  if (SDFisBoolElemsEmpty(tail)) {
    return SDFmakeBoolElemsSingle(head);
  }
  return (SDFBoolElems)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeList1((ATerm)ATmakeInt(59)))), (ATerm)ATmakeAppl1(SDFafun16, (ATerm)ATmakeAppl0(SDFafun30)), (ATerm)ATmakeAppl0(SDFafun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(59)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool SDFisEqualOptLayout(SDFOptLayout arg0, SDFOptLayout arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualLayout(SDFLayout arg0, SDFLayout arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualLexLayoutList(SDFLexLayoutList arg0, SDFLexLayoutList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualStart(SDFStart arg0, SDFStart arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualLexLayout(SDFLexLayout arg0, SDFLexLayout arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualBool(SDFBool arg0, SDFBool arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualBoolList(SDFBoolList arg0, SDFBoolList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualBoolElems(SDFBoolElems arg0, SDFBoolElems arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  SDFOptLayout accessors */

/*{{{  ATbool SDFisValidOptLayout(SDFOptLayout arg) */

ATbool SDFisValidOptLayout(SDFOptLayout arg)
{
  if (SDFisOptLayoutAbsent(arg)) {
    return ATtrue;
  }
  else if (SDFisOptLayoutPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisOptLayoutAbsent(SDFOptLayout arg) */

inline ATbool SDFisOptLayoutAbsent(SDFOptLayout arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternOptLayoutAbsent);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisOptLayoutPresent(SDFOptLayout arg) */

inline ATbool SDFisOptLayoutPresent(SDFOptLayout arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternOptLayoutPresent, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDFhasOptLayoutLayout(SDFOptLayout arg) */

ATbool SDFhasOptLayoutLayout(SDFOptLayout arg)
{
  if (SDFisOptLayoutPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFLayout SDFgetOptLayoutLayout(SDFOptLayout arg) */

SDFLayout SDFgetOptLayoutLayout(SDFOptLayout arg)
{
  
    return (SDFLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDFOptLayout SDFsetOptLayoutLayout(SDFOptLayout arg, SDFLayout layout) */

SDFOptLayout SDFsetOptLayoutLayout(SDFOptLayout arg, SDFLayout layout)
{
  if (SDFisOptLayoutPresent(arg)) {
    return (SDFOptLayout)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) layout), 0), 1);
  }

  ATabort("OptLayout has no Layout: %t\n", arg);
  return (SDFOptLayout)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFLayout accessors */

/*{{{  ATbool SDFisValidLayout(SDFLayout arg) */

ATbool SDFisValidLayout(SDFLayout arg)
{
  if (SDFisLayoutLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisLayoutLexToCf(SDFLayout arg) */

inline ATbool SDFisLayoutLexToCf(SDFLayout arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDFpatternLayoutLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDFhasLayoutList(SDFLayout arg) */

ATbool SDFhasLayoutList(SDFLayout arg)
{
  if (SDFisLayoutLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFLexLayoutList SDFgetLayoutList(SDFLayout arg) */

SDFLexLayoutList SDFgetLayoutList(SDFLayout arg)
{
  
    return (SDFLexLayoutList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDFLayout SDFsetLayoutList(SDFLayout arg, SDFLexLayoutList list) */

SDFLayout SDFsetLayoutList(SDFLayout arg, SDFLexLayoutList list)
{
  if (SDFisLayoutLexToCf(arg)) {
    return (SDFLayout)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Layout has no List: %t\n", arg);
  return (SDFLayout)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFLexLayoutList accessors */

/*{{{  ATbool SDFisValidLexLayoutList(SDFLexLayoutList arg) */

ATbool SDFisValidLexLayoutList(SDFLexLayoutList arg)
{
  if (SDFisLexLayoutListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDFisLexLayoutListSingle(arg)) {
    return ATtrue;
  }
  else if (SDFisLexLayoutListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisLexLayoutListEmpty(SDFLexLayoutList arg) */

inline ATbool SDFisLexLayoutListEmpty(SDFLexLayoutList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDFpatternLexLayoutListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDFisLexLayoutListSingle(SDFLexLayoutList arg) */

inline ATbool SDFisLexLayoutListSingle(SDFLexLayoutList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternLexLayoutListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisLexLayoutListMany(SDFLexLayoutList arg) */

inline ATbool SDFisLexLayoutListMany(SDFLexLayoutList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternLexLayoutListMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDFhasLexLayoutListHead(SDFLexLayoutList arg) */

ATbool SDFhasLexLayoutListHead(SDFLexLayoutList arg)
{
  if (SDFisLexLayoutListSingle(arg)) {
    return ATtrue;
  }
  else if (SDFisLexLayoutListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasLexLayoutListTail(SDFLexLayoutList arg) */

ATbool SDFhasLexLayoutListTail(SDFLexLayoutList arg)
{
  if (SDFisLexLayoutListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFLexLayout SDFgetLexLayoutListHead(SDFLexLayoutList arg) */

SDFLexLayout SDFgetLexLayoutListHead(SDFLexLayoutList arg)
{
  if (SDFisLexLayoutListSingle(arg)) {
    return (SDFLexLayout)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDFLexLayout)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDFLexLayoutList SDFgetLexLayoutListTail(SDFLexLayoutList arg) */

SDFLexLayoutList SDFgetLexLayoutListTail(SDFLexLayoutList arg)
{
  
    return (SDFLexLayoutList)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  SDFLexLayoutList SDFsetLexLayoutListHead(SDFLexLayoutList arg, SDFLexLayout head) */

SDFLexLayoutList SDFsetLexLayoutListHead(SDFLexLayoutList arg, SDFLexLayout head)
{
  if (SDFisLexLayoutListSingle(arg)) {
    return (SDFLexLayoutList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDFisLexLayoutListMany(arg)) {
    return (SDFLexLayoutList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("LexLayoutList has no Head: %t\n", arg);
  return (SDFLexLayoutList)NULL;
}

/*}}}  */
/*{{{  SDFLexLayoutList SDFsetLexLayoutListTail(SDFLexLayoutList arg, SDFLexLayoutList tail) */

SDFLexLayoutList SDFsetLexLayoutListTail(SDFLexLayoutList arg, SDFLexLayoutList tail)
{
  if (SDFisLexLayoutListMany(arg)) {
    return (SDFLexLayoutList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("LexLayoutList has no Tail: %t\n", arg);
  return (SDFLexLayoutList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFStart accessors */

/*{{{  ATbool SDFisValidStart(SDFStart arg) */

ATbool SDFisValidStart(SDFStart arg)
{
  if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  else if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisStartBool(SDFStart arg) */

inline ATbool SDFisStartBool(SDFStart arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternStartBool, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisStartBoolList(SDFStart arg) */

inline ATbool SDFisStartBoolList(SDFStart arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternStartBoolList, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDFhasStartWsBefore(SDFStart arg) */

ATbool SDFhasStartWsBefore(SDFStart arg)
{
  if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  else if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasStartTopBool(SDFStart arg) */

ATbool SDFhasStartTopBool(SDFStart arg)
{
  if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasStartWsAfter(SDFStart arg) */

ATbool SDFhasStartWsAfter(SDFStart arg)
{
  if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  else if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasStartAmbCnt(SDFStart arg) */

ATbool SDFhasStartAmbCnt(SDFStart arg)
{
  if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  else if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasStartTopBoolList(SDFStart arg) */

ATbool SDFhasStartTopBoolList(SDFStart arg)
{
  if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetStartWsBefore(SDFStart arg) */

SDFOptLayout SDFgetStartWsBefore(SDFStart arg)
{
  if (SDFisStartBool(arg)) {
    return (SDFOptLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1));
  }
  else 
    return (SDFOptLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1));
}

/*}}}  */
/*{{{  SDFBool SDFgetStartTopBool(SDFStart arg) */

SDFBool SDFgetStartTopBool(SDFStart arg)
{
  
    return (SDFBool)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 1);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetStartWsAfter(SDFStart arg) */

SDFOptLayout SDFgetStartWsAfter(SDFStart arg)
{
  if (SDFisStartBool(arg)) {
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 2);
  }
  else 
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 2);
}

/*}}}  */
/*{{{  int SDFgetStartAmbCnt(SDFStart arg) */

int SDFgetStartAmbCnt(SDFStart arg)
{
  if (SDFisStartBool(arg)) {
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDFBoolList SDFgetStartTopBoolList(SDFStart arg) */

SDFBoolList SDFgetStartTopBoolList(SDFStart arg)
{
  
    return (SDFBoolList)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 1);
}

/*}}}  */
/*{{{  SDFStart SDFsetStartWsBefore(SDFStart arg, SDFOptLayout wsBefore) */

SDFStart SDFsetStartWsBefore(SDFStart arg, SDFOptLayout wsBefore)
{
  if (SDFisStartBool(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsBefore), 0), 1), 0);
  }
  else if (SDFisStartBoolList(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsBefore), 0), 1), 0);
  }

  ATabort("Start has no WsBefore: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFsetStartTopBool(SDFStart arg, SDFBool topBool) */

SDFStart SDFsetStartTopBool(SDFStart arg, SDFBool topBool)
{
  if (SDFisStartBool(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) topBool), 1), 1), 0);
  }

  ATabort("Start has no TopBool: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFsetStartWsAfter(SDFStart arg, SDFOptLayout wsAfter) */

SDFStart SDFsetStartWsAfter(SDFStart arg, SDFOptLayout wsAfter)
{
  if (SDFisStartBool(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsAfter), 2), 1), 0);
  }
  else if (SDFisStartBoolList(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsAfter), 2), 1), 0);
  }

  ATabort("Start has no WsAfter: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFsetStartAmbCnt(SDFStart arg, int ambCnt) */

SDFStart SDFsetStartAmbCnt(SDFStart arg, int ambCnt)
{
  if (SDFisStartBool(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(ambCnt)), 1);
  }
  else if (SDFisStartBoolList(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(ambCnt)), 1);
  }

  ATabort("Start has no AmbCnt: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFsetStartTopBoolList(SDFStart arg, SDFBoolList topBoolList) */

SDFStart SDFsetStartTopBoolList(SDFStart arg, SDFBoolList topBoolList)
{
  if (SDFisStartBoolList(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) topBoolList), 1), 1), 0);
  }

  ATabort("Start has no TopBoolList: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFLexLayout accessors */

/*{{{  ATbool SDFisValidLexLayout(SDFLexLayout arg) */

ATbool SDFisValidLexLayout(SDFLexLayout arg)
{
  if (SDFisLexLayoutDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisLexLayoutDefault(SDFLexLayout arg) */

inline ATbool SDFisLexLayoutDefault(SDFLexLayout arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDFpatternLexLayoutDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDFhasLexLayoutCh(SDFLexLayout arg) */

ATbool SDFhasLexLayoutCh(SDFLexLayout arg)
{
  if (SDFisLexLayoutDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char SDFgetLexLayoutCh(SDFLexLayout arg) */

char SDFgetLexLayoutCh(SDFLexLayout arg)
{
  
    return (char)SDFcharToByte((ATerm)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)));
}

/*}}}  */
/*{{{  SDFLexLayout SDFsetLexLayoutCh(SDFLexLayout arg, char ch) */

SDFLexLayout SDFsetLexLayoutCh(SDFLexLayout arg, char ch)
{
  if (SDFisLexLayoutDefault(arg)) {
    return (SDFLexLayout)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDFbyteToChar(ch))), 0), 1);
  }

  ATabort("LexLayout has no Ch: %t\n", arg);
  return (SDFLexLayout)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFBool accessors */

/*{{{  ATbool SDFisValidBool(SDFBool arg) */

ATbool SDFisValidBool(SDFBool arg)
{
  if (SDFisBoolTrue(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolFalse(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolTrue(SDFBool arg) */

inline ATbool SDFisBoolTrue(SDFBool arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolTrue);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolFalse(SDFBool arg) */

inline ATbool SDFisBoolFalse(SDFBool arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolFalse);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolAnd(SDFBool arg) */

inline ATbool SDFisBoolAnd(SDFBool arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolAnd, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolOr(SDFBool arg) */

inline ATbool SDFisBoolOr(SDFBool arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolOr, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDFhasBoolLeft(SDFBool arg) */

ATbool SDFhasBoolLeft(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolWsAfterLeft(SDFBool arg) */

ATbool SDFhasBoolWsAfterLeft(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolWsAfterAmp(SDFBool arg) */

ATbool SDFhasBoolWsAfterAmp(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolRight(SDFBool arg) */

ATbool SDFhasBoolRight(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolWsAfterBar(SDFBool arg) */

ATbool SDFhasBoolWsAfterBar(SDFBool arg)
{
  if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFBool SDFgetBoolLeft(SDFBool arg) */

SDFBool SDFgetBoolLeft(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (SDFBool)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolWsAfterLeft(SDFBool arg) */

SDFOptLayout SDFgetBoolWsAfterLeft(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolWsAfterAmp(SDFBool arg) */

SDFOptLayout SDFgetBoolWsAfterAmp(SDFBool arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDFBool SDFgetBoolRight(SDFBool arg) */

SDFBool SDFgetBoolRight(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else 
    return (SDFBool)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolWsAfterBar(SDFBool arg) */

SDFOptLayout SDFgetBoolWsAfterBar(SDFBool arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolLeft(SDFBool arg, SDFBool left) */

SDFBool SDFsetBoolLeft(SDFBool arg, SDFBool left)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }
  else if (SDFisBoolOr(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }

  ATabort("Bool has no Left: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolWsAfterLeft(SDFBool arg, SDFOptLayout wsAfterLeft) */

SDFBool SDFsetBoolWsAfterLeft(SDFBool arg, SDFOptLayout wsAfterLeft)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }
  else if (SDFisBoolOr(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }

  ATabort("Bool has no WsAfterLeft: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolWsAfterAmp(SDFBool arg, SDFOptLayout wsAfterAmp) */

SDFBool SDFsetBoolWsAfterAmp(SDFBool arg, SDFOptLayout wsAfterAmp)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterAmp), 3), 1);
  }

  ATabort("Bool has no WsAfterAmp: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolRight(SDFBool arg, SDFBool right) */

SDFBool SDFsetBoolRight(SDFBool arg, SDFBool right)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }
  else if (SDFisBoolOr(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }

  ATabort("Bool has no Right: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolWsAfterBar(SDFBool arg, SDFOptLayout wsAfterBar) */

SDFBool SDFsetBoolWsAfterBar(SDFBool arg, SDFOptLayout wsAfterBar)
{
  if (SDFisBoolOr(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBar), 3), 1);
  }

  ATabort("Bool has no WsAfterBar: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFBoolList accessors */

/*{{{  ATbool SDFisValidBoolList(SDFBoolList arg) */

ATbool SDFisValidBoolList(SDFBoolList arg)
{
  if (SDFisBoolListElems(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolListElems(SDFBoolList arg) */

inline ATbool SDFisBoolListElems(SDFBoolList arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDFpatternBoolListElems, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolListWsAfterBracketOpen(SDFBoolList arg) */

ATbool SDFhasBoolListWsAfterBracketOpen(SDFBoolList arg)
{
  if (SDFisBoolListElems(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolListElems(SDFBoolList arg) */

ATbool SDFhasBoolListElems(SDFBoolList arg)
{
  if (SDFisBoolListElems(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolListWsAfterElems(SDFBoolList arg) */

ATbool SDFhasBoolListWsAfterElems(SDFBoolList arg)
{
  if (SDFisBoolListElems(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolListWsAfterBracketOpen(SDFBoolList arg) */

SDFOptLayout SDFgetBoolListWsAfterBracketOpen(SDFBoolList arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDFBoolElems SDFgetBoolListElems(SDFBoolList arg) */

SDFBoolElems SDFgetBoolListElems(SDFBoolList arg)
{
  
    return (SDFBoolElems)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), 1);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolListWsAfterElems(SDFBoolList arg) */

SDFOptLayout SDFgetBoolListWsAfterElems(SDFBoolList arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDFBoolList SDFsetBoolListWsAfterBracketOpen(SDFBoolList arg, SDFOptLayout wsAfterBracketOpen) */

SDFBoolList SDFsetBoolListWsAfterBracketOpen(SDFBoolList arg, SDFOptLayout wsAfterBracketOpen)
{
  if (SDFisBoolListElems(arg)) {
    return (SDFBoolList)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBracketOpen), 1), 1);
  }

  ATabort("BoolList has no WsAfterBracketOpen: %t\n", arg);
  return (SDFBoolList)NULL;
}

/*}}}  */
/*{{{  SDFBoolList SDFsetBoolListElems(SDFBoolList arg, SDFBoolElems elems) */

SDFBoolList SDFsetBoolListElems(SDFBoolList arg, SDFBoolElems elems)
{
  if (SDFisBoolListElems(arg)) {
    return (SDFBoolList)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), (ATerm)((ATerm) elems), 1), 2), 1);
  }

  ATabort("BoolList has no Elems: %t\n", arg);
  return (SDFBoolList)NULL;
}

/*}}}  */
/*{{{  SDFBoolList SDFsetBoolListWsAfterElems(SDFBoolList arg, SDFOptLayout wsAfterElems) */

SDFBoolList SDFsetBoolListWsAfterElems(SDFBoolList arg, SDFOptLayout wsAfterElems)
{
  if (SDFisBoolListElems(arg)) {
    return (SDFBoolList)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterElems), 3), 1);
  }

  ATabort("BoolList has no WsAfterElems: %t\n", arg);
  return (SDFBoolList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFBoolElems accessors */

/*{{{  ATbool SDFisValidBoolElems(SDFBoolElems arg) */

ATbool SDFisValidBoolElems(SDFBoolElems arg)
{
  if (SDFisBoolElemsEmpty(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolElemsSingle(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolElemsEmpty(SDFBoolElems arg) */

inline ATbool SDFisBoolElemsEmpty(SDFBoolElems arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDFpatternBoolElemsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolElemsSingle(SDFBoolElems arg) */

inline ATbool SDFisBoolElemsSingle(SDFBoolElems arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolElemsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolElemsMany(SDFBoolElems arg) */

inline ATbool SDFisBoolElemsMany(SDFBoolElems arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolElemsMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDFhasBoolElemsHead(SDFBoolElems arg) */

ATbool SDFhasBoolElemsHead(SDFBoolElems arg)
{
  if (SDFisBoolElemsSingle(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolElemsWsAfterHead(SDFBoolElems arg) */

ATbool SDFhasBoolElemsWsAfterHead(SDFBoolElems arg)
{
  if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolElemsWsAfterSep(SDFBoolElems arg) */

ATbool SDFhasBoolElemsWsAfterSep(SDFBoolElems arg)
{
  if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolElemsTail(SDFBoolElems arg) */

ATbool SDFhasBoolElemsTail(SDFBoolElems arg)
{
  if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFBoolElems SDFgetBoolElemsTail(SDFBoolElems arg) */

SDFBoolElems SDFgetBoolElemsTail(SDFBoolElems arg)
{
  assert(!SDFisBoolElemsEmpty(arg) && "getTail on an empty list");
  if (SDFisBoolElemsSingle(arg)) {
    return (SDFBoolElems) SDFmakeBoolElemsEmpty();
  }
  else {
  
    return (SDFBoolElems)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDFBool SDFgetBoolElemsHead(SDFBoolElems arg) */

SDFBool SDFgetBoolElemsHead(SDFBoolElems arg)
{
  if (SDFisBoolElemsSingle(arg)) {
    return (SDFBool)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDFBool)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolElemsWsAfterHead(SDFBoolElems arg) */

SDFOptLayout SDFgetBoolElemsWsAfterHead(SDFBoolElems arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolElemsWsAfterSep(SDFBoolElems arg) */

SDFOptLayout SDFgetBoolElemsWsAfterSep(SDFBoolElems arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDFBoolElems SDFsetBoolElemsHead(SDFBoolElems arg, SDFBool head) */

SDFBoolElems SDFsetBoolElemsHead(SDFBoolElems arg, SDFBool head)
{
  if (SDFisBoolElemsSingle(arg)) {
    return (SDFBoolElems)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDFisBoolElemsMany(arg)) {
    return (SDFBoolElems)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("BoolElems has no Head: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */
/*{{{  SDFBoolElems SDFsetBoolElemsWsAfterHead(SDFBoolElems arg, SDFOptLayout wsAfterHead) */

SDFBoolElems SDFsetBoolElemsWsAfterHead(SDFBoolElems arg, SDFOptLayout wsAfterHead)
{
  if (SDFisBoolElemsMany(arg)) {
    return (SDFBoolElems)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("BoolElems has no WsAfterHead: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */
/*{{{  SDFBoolElems SDFsetBoolElemsWsAfterSep(SDFBoolElems arg, SDFOptLayout wsAfterSep) */

SDFBoolElems SDFsetBoolElemsWsAfterSep(SDFBoolElems arg, SDFOptLayout wsAfterSep)
{
  if (SDFisBoolElemsMany(arg)) {
    return (SDFBoolElems)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("BoolElems has no WsAfterSep: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */
/*{{{  SDFBoolElems SDFsetBoolElemsTail(SDFBoolElems arg, SDFBoolElems tail) */

SDFBoolElems SDFsetBoolElemsTail(SDFBoolElems arg, SDFBoolElems tail)
{
  if (SDFisBoolElemsMany(arg)) {
    return (SDFBoolElems)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("BoolElems has no Tail: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  SDFOptLayout SDFvisitOptLayout(SDFOptLayout arg, SDFLayout (*acceptLayout)(SDFLayout)) */

SDFOptLayout SDFvisitOptLayout(SDFOptLayout arg, SDFLayout (*acceptLayout)(SDFLayout))
{
  if (SDFisOptLayoutAbsent(arg)) {
    return SDFmakeOptLayoutAbsent();
  }
  if (SDFisOptLayoutPresent(arg)) {
    return SDFmakeOptLayoutPresent(
        acceptLayout ? acceptLayout(SDFgetOptLayoutLayout(arg)) : SDFgetOptLayoutLayout(arg));
  }
  ATabort("not a OptLayout: %t\n", arg);
  return (SDFOptLayout)NULL;
}

/*}}}  */
/*{{{  SDFLayout SDFvisitLayout(SDFLayout arg, SDFLexLayoutList (*acceptList)(SDFLexLayoutList)) */

SDFLayout SDFvisitLayout(SDFLayout arg, SDFLexLayoutList (*acceptList)(SDFLexLayoutList))
{
  if (SDFisLayoutLexToCf(arg)) {
    return SDFmakeLayoutLexToCf(
        acceptList ? acceptList(SDFgetLayoutList(arg)) : SDFgetLayoutList(arg));
  }
  ATabort("not a Layout: %t\n", arg);
  return (SDFLayout)NULL;
}

/*}}}  */
/*{{{  SDFLexLayoutList SDFvisitLexLayoutList(SDFLexLayoutList arg, SDFLexLayout (*acceptHead)(SDFLexLayout)) */

SDFLexLayoutList SDFvisitLexLayoutList(SDFLexLayoutList arg, SDFLexLayout (*acceptHead)(SDFLexLayout))
{
  if (SDFisLexLayoutListEmpty(arg)) {
    return SDFmakeLexLayoutListEmpty();
  }
  if (SDFisLexLayoutListSingle(arg)) {
    return SDFmakeLexLayoutListSingle(
        acceptHead ? acceptHead(SDFgetLexLayoutListHead(arg)) : SDFgetLexLayoutListHead(arg));
  }
  if (SDFisLexLayoutListMany(arg)) {
    return SDFmakeLexLayoutListMany(
        acceptHead ? acceptHead(SDFgetLexLayoutListHead(arg)) : SDFgetLexLayoutListHead(arg),
        SDFvisitLexLayoutList(SDFgetLexLayoutListTail(arg), acceptHead));
  }
  ATabort("not a LexLayoutList: %t\n", arg);
  return (SDFLexLayoutList)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFvisitStart(SDFStart arg, SDFOptLayout (*acceptWsBefore)(SDFOptLayout), SDFBool (*acceptTopBool)(SDFBool), SDFOptLayout (*acceptWsAfter)(SDFOptLayout), int (*acceptAmbCnt)(int), SDFBoolList (*acceptTopBoolList)(SDFBoolList)) */

SDFStart SDFvisitStart(SDFStart arg, SDFOptLayout (*acceptWsBefore)(SDFOptLayout), SDFBool (*acceptTopBool)(SDFBool), SDFOptLayout (*acceptWsAfter)(SDFOptLayout), int (*acceptAmbCnt)(int), SDFBoolList (*acceptTopBoolList)(SDFBoolList))
{
  if (SDFisStartBool(arg)) {
    return SDFmakeStartBool(
        acceptWsBefore ? acceptWsBefore(SDFgetStartWsBefore(arg)) : SDFgetStartWsBefore(arg),
        acceptTopBool ? acceptTopBool(SDFgetStartTopBool(arg)) : SDFgetStartTopBool(arg),
        acceptWsAfter ? acceptWsAfter(SDFgetStartWsAfter(arg)) : SDFgetStartWsAfter(arg),
        acceptAmbCnt ? acceptAmbCnt(SDFgetStartAmbCnt(arg)) : SDFgetStartAmbCnt(arg));
  }
  if (SDFisStartBoolList(arg)) {
    return SDFmakeStartBoolList(
        acceptWsBefore ? acceptWsBefore(SDFgetStartWsBefore(arg)) : SDFgetStartWsBefore(arg),
        acceptTopBoolList ? acceptTopBoolList(SDFgetStartTopBoolList(arg)) : SDFgetStartTopBoolList(arg),
        acceptWsAfter ? acceptWsAfter(SDFgetStartWsAfter(arg)) : SDFgetStartWsAfter(arg),
        acceptAmbCnt ? acceptAmbCnt(SDFgetStartAmbCnt(arg)) : SDFgetStartAmbCnt(arg));
  }
  ATabort("not a Start: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFLexLayout SDFvisitLexLayout(SDFLexLayout arg, char (*acceptCh)(char)) */

SDFLexLayout SDFvisitLexLayout(SDFLexLayout arg, char (*acceptCh)(char))
{
  if (SDFisLexLayoutDefault(arg)) {
    return SDFmakeLexLayoutDefault(
        acceptCh ? acceptCh(SDFgetLexLayoutCh(arg)) : SDFgetLexLayoutCh(arg));
  }
  ATabort("not a LexLayout: %t\n", arg);
  return (SDFLexLayout)NULL;
}

/*}}}  */
/*{{{  SDFBool SDFvisitBool(SDFBool arg, SDFOptLayout (*acceptWsAfterLeft)(SDFOptLayout), SDFOptLayout (*acceptWsAfterAmp)(SDFOptLayout), SDFOptLayout (*acceptWsAfterBar)(SDFOptLayout)) */

SDFBool SDFvisitBool(SDFBool arg, SDFOptLayout (*acceptWsAfterLeft)(SDFOptLayout), SDFOptLayout (*acceptWsAfterAmp)(SDFOptLayout), SDFOptLayout (*acceptWsAfterBar)(SDFOptLayout))
{
  if (SDFisBoolTrue(arg)) {
    return SDFmakeBoolTrue();
  }
  if (SDFisBoolFalse(arg)) {
    return SDFmakeBoolFalse();
  }
  if (SDFisBoolAnd(arg)) {
    return SDFmakeBoolAnd(
        SDFvisitBool(SDFgetBoolLeft(arg), acceptWsAfterLeft, acceptWsAfterAmp, acceptWsAfterBar),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDFgetBoolWsAfterLeft(arg)) : SDFgetBoolWsAfterLeft(arg),
        acceptWsAfterAmp ? acceptWsAfterAmp(SDFgetBoolWsAfterAmp(arg)) : SDFgetBoolWsAfterAmp(arg),
        SDFvisitBool(SDFgetBoolRight(arg), acceptWsAfterLeft, acceptWsAfterAmp, acceptWsAfterBar));
  }
  if (SDFisBoolOr(arg)) {
    return SDFmakeBoolOr(
        SDFvisitBool(SDFgetBoolLeft(arg), acceptWsAfterLeft, acceptWsAfterAmp, acceptWsAfterBar),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDFgetBoolWsAfterLeft(arg)) : SDFgetBoolWsAfterLeft(arg),
        acceptWsAfterBar ? acceptWsAfterBar(SDFgetBoolWsAfterBar(arg)) : SDFgetBoolWsAfterBar(arg),
        SDFvisitBool(SDFgetBoolRight(arg), acceptWsAfterLeft, acceptWsAfterAmp, acceptWsAfterBar));
  }
  ATabort("not a Bool: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBoolList SDFvisitBoolList(SDFBoolList arg, SDFOptLayout (*acceptWsAfterBracketOpen)(SDFOptLayout), SDFBoolElems (*acceptElems)(SDFBoolElems), SDFOptLayout (*acceptWsAfterElems)(SDFOptLayout)) */

SDFBoolList SDFvisitBoolList(SDFBoolList arg, SDFOptLayout (*acceptWsAfterBracketOpen)(SDFOptLayout), SDFBoolElems (*acceptElems)(SDFBoolElems), SDFOptLayout (*acceptWsAfterElems)(SDFOptLayout))
{
  if (SDFisBoolListElems(arg)) {
    return SDFmakeBoolListElems(
        acceptWsAfterBracketOpen ? acceptWsAfterBracketOpen(SDFgetBoolListWsAfterBracketOpen(arg)) : SDFgetBoolListWsAfterBracketOpen(arg),
        acceptElems ? acceptElems(SDFgetBoolListElems(arg)) : SDFgetBoolListElems(arg),
        acceptWsAfterElems ? acceptWsAfterElems(SDFgetBoolListWsAfterElems(arg)) : SDFgetBoolListWsAfterElems(arg));
  }
  ATabort("not a BoolList: %t\n", arg);
  return (SDFBoolList)NULL;
}

/*}}}  */
/*{{{  SDFBoolElems SDFvisitBoolElems(SDFBoolElems arg, SDFBool (*acceptHead)(SDFBool), SDFOptLayout (*acceptWsAfterHead)(SDFOptLayout), SDFOptLayout (*acceptWsAfterSep)(SDFOptLayout)) */

SDFBoolElems SDFvisitBoolElems(SDFBoolElems arg, SDFBool (*acceptHead)(SDFBool), SDFOptLayout (*acceptWsAfterHead)(SDFOptLayout), SDFOptLayout (*acceptWsAfterSep)(SDFOptLayout))
{
  if (SDFisBoolElemsEmpty(arg)) {
    return SDFmakeBoolElemsEmpty();
  }
  if (SDFisBoolElemsSingle(arg)) {
    return SDFmakeBoolElemsSingle(
        acceptHead ? acceptHead(SDFgetBoolElemsHead(arg)) : SDFgetBoolElemsHead(arg));
  }
  if (SDFisBoolElemsMany(arg)) {
    return SDFmakeBoolElemsMany(
        acceptHead ? acceptHead(SDFgetBoolElemsHead(arg)) : SDFgetBoolElemsHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDFgetBoolElemsWsAfterHead(arg)) : SDFgetBoolElemsWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDFgetBoolElemsWsAfterSep(arg)) : SDFgetBoolElemsWsAfterSep(arg),
        SDFvisitBoolElems(SDFgetBoolElemsTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a BoolElems: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */

/*}}}  */
